#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#if defined(__WIN32__) || defined(_WIN32)
#include <windows.h>
#endif

#include "data_pronouncer.h"


extern char modelPath[];


extern void makeModel(void);


static long rn;

static void rseed(long s)
{
    rn = s;
}

static double rand01(void)
{
    rn = (69069 * rn) % 0x80000000;
    return rn / (double)0x80000000;
}


void getProbabilities(int p, int s, double *probs)
{
    while (p--)
    {
	int i = s - 1;
	double last = 1.0;

	while (i--)
	    last -= (*probs++ = last * rand01());
	*probs++ = last;
    }
}


void getUtilities(int p, double *utils)
{
    while (p--)
	*utils++ = rand01();
}


int runTest(int argc, char **argv)
{
    int i, ntrials = 100;
    long seed = time(NULL);
    int advice;
#if defined(__WIN32__) || defined(_WIN32)
    LARGE_INTEGER freq, start, stop;
#else
    struct timeval start, stop;
#endif
    double m;
    double sum = 0.0;
    double sqsum = 0.0;
    int verbose = 0;

    if (argc > 1)
	ntrials = atoi(argv[1]);
    if (argc > 2)
	seed = atoi(argv[2]);
    if (argc > 3)
	verbose = (argv[3][0] == 'v');
    printf("> %s %d %ld\n", argv[0], ntrials, seed);

    rseed(seed);
    initializePronouncer();
    loadModel(modelPath);

#if defined(__WIN32__) || defined(_WIN32)
    QueryPerformanceFrequency(&freq);
#endif
    for (i = 0; i < ntrials; i++)
    {
	makeModel();
#if defined(__WIN32__) || defined(_WIN32)
        QueryPerformanceCounter(&start);
#else
	gettimeofday(&start, NULL);
#endif
	advice = evaluateModel(NULL, 0);
#if defined(__WIN32__) || defined(_WIN32)
        QueryPerformanceCounter(&stop);
        m = 1000.0 * ((stop.QuadPart - start.QuadPart) / (double)freq.QuadPart);
#else
	gettimeofday(&stop, NULL);
	if (stop.tv_usec < start.tv_usec)
	{
	    stop.tv_usec += 1000000;
	    start.tv_sec += 1;
	}
	m = 1000.0 * (stop.tv_sec - start.tv_sec) +
	    (stop.tv_usec - start.tv_usec) / 1000.0;
#endif
	if (verbose)
	    printf("%7.3f (%d)\n", m, advice);
	sum += m;
	sqsum += m * m;
    }
    printf("mean is %f ms\n", sum / ntrials);
    printf("deviation is %f\n", sqrt((sqsum - sum * sum / ntrials) / ntrials));

    finalizePronouncer();

    return 0;
}

