package decisionanalysis;

import java.io.Serializable;
import java.io.StreamTokenizer;
import java.io.IOException;

/**
 * A decision tree node.
 *
 * @author H&aring;kan L. Younes
 */
abstract class DTNode extends Object implements Serializable
{
    /**
     * The tree that this node belongs to.
     */
    protected DecisionTree tree;


    /**
     * Reads a node from the given source.
     *
     * @param st
     *  source for node description.
     * @param tree
     *  decision tree that the node belongs to.
     * @return the read node.
     * @exception IOException
     *  if an I/O error occurs.
     */
    protected static DTNode readFrom(StreamTokenizer st, DecisionTree tree)
	 throws IOException
    {
	if (st.nextToken() != StreamTokenizer.TT_WORD)
	{
	    throw new IOException("illegal token: " + st);
	}
	if (st.sval.equals("DECISION_NODE"))
	{
	    return DTDecisionNode.readFrom(st, tree);
	}
	else if (st.sval.equals("CHANCE_NODE"))
	{
	    return DTChanceNode.readFrom(st, tree);
	}
	else if (st.sval.equals("TERMINAL_NODE"))
	{
	    return DTTerminalNode.readFrom(st, tree);
	}
	else
	{
	    throw new IOException("illegal node type: " + st.sval);
	}
    }


    /**
     * Constructs a decision tree node.
     *
     * @param tree
     *  decision tree that the node belongs to.
     */
    protected DTNode(DecisionTree tree)
    {
	this.tree = tree;
    }


    /**
     * Returns the expected utility for this node.
     *
     * @return the expected utility for this node.
     */
    public abstract double expectedUtility();

    /**
     * Returns a string representation of this node.
     *
     * @param depth
     *  depth of node in decision tree.
     * @return a string representation of this node.
     */
    public abstract String toString(int depth);
}
